CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCYCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      program Li_atom
C  QMC main program of Li atom
       use highlevel
       use midlevel
       use orbital
       use determinant
       use random
       use jastrow, JEXP=>JASEXPATOM
       use observables
       use output
       implicit none
       integer          :: i,ii,hi,j,jj,islap,n1
       real(dp)         :: q,rd,rannumb
C  Open output data files
       call INITOUT
C  Fill an array with a set of random numbers to be available;
C  contained in M_random.f.
C  RANDNAME = name of the random generator
       RANDNAME="random generator from REC_PJN           "
C       RANDNAME="random generator from TAO               "
C       RANDNAME="random generator from G95               "
C       RANDNAME="random generator from F90/95            "
       MCPRE = 100000
       MCMAX = 20000000
       MAXA  = 100000
       write(*,*)'Prerun and main run number of steps, MCPRE,MCMAX=',
     &                                                 MCPRE,MCMAX
       call INITRAN
C  Set the initial electron positions;
C  contained in M_random.f
       call INITRANNUMB
C  Associate electron with single particle wavefunction, normalize
C  wavefunction, and determine initial wavefunction matrix;
C  contained in M_orbital_Li.f
       SLAP(1)=0.94_dp
       SLAP(2)=0.93_dp
       write(*,*)'Slater parameter loop, initial and increment,
     &            SLAP(1),SLAP(2)=',SLAP(1),SLAP(2)
       ljas: do n1=1,15
       CJAS=0.000000001_dp+n1*0.2_dp
       write(*,*)'CJAS=',CJAS
       call INITORB
C  Set initial values for Jastrow parameters
       call INITJAS
C  Set initial values for inverse matrices;
C  contained in M_determinant.f
       call INITDET
C  Set the initial values of the MC run;
C  contained in M_observables.f
       call INITOBS
       STEPMAX = 0.5_dp
       write(*,*)'Maximum step interval= +-0.5*STEPMAX, STEPMAX=',
     &  STEPMAX

C
C  MC loop:
      MCOUNT = 0
C  First, the prerun for thermalizing
      lrun:do IMCR=1,MCMAX
C  Start the main run with statistical sampling of the observables if
C  the number of prerun steps is reached. Initialize the observables
C  and their statistics
       if ( IMCR == MCPRE) then
        write(*,*)'prerun: MCPRE= ',MCPRE,' acc. ratio = ',
     &           100.*DBLE(MCOUNT)/DBLE(NE*(MCPRE)),' %  '
        call INITRANOBS
       end if
       if (IMCR >= MCPRE) IMC=IMCR-MCPRE+1
C
C  Inner loop on electron index:
       lelrun:do IE=1,NE
        IES=1
        IEES=IE
        if (IE > NES(1)) IES=2
        hi=(IES-1)*NES(1)
        IEES=IE-hi
        IENS=NES(IES)
        do i=1,3
C  Shift position at random within +-STEPMAX/2
         call GENRAN(rannumb)
         rd = (rannumb-0.5_dp)*STEPMAX
         RENEW(i,IE) = RE(i,IE)+rd
        end do
C  Calculate Jastrow factor quantities
        call JEXP
C  Calculate single particle wavefunction part
        call ORBWAV(RENEW(1:3,IE),PSINEW)
        do ii=1,IENS
         PSIMAT(ii,IEES,IES)=PSINEW(NELORB(hi+ii))
        end do
        call SLAQUOT(AOLD(1:IENS,1:IENS,IES),
     &               PSIMAT(1:IENS,1:IENS,IES),QD(IES))
C  Test on acceptance
        q = (QD(IES)*QJ(IES))**2
        if (q < 1) then
         call GENRAN(rannumb)
         MCSTEP = (rannumb < q)
        else
         MCSTEP = .true.
        end if
C  Calculates observables
        if (MCRUN) then
         call OBSERV
        end if
C  Update for acceptance or not
        lmcstep:if (MCSTEP) then
         RE(1:3,IE) = RENEW(1:3,IE)
         call SLASM(QD(IES),AOLD(1:IENS,1:IENS,IES),
     &         PSIMAT(1:IENS,1:IENS,IES),ANEW(1:IENS,1:IENS,IES))
         AOLD(1:IENS,1:IENS,IES) = ANEW(1:IENS,1:IENS,IES)
         MCOUNT = MCOUNT + 1
        end if lmcstep
C  Calculate statistical results of observables per electron
        if (MCRUN) call OBSERVSTATEL
       end do lelrun
C
C  Calculate statistical results of observables for all electrons
       if (MCRUN) call OBSERVSTATALL
      end do lrun
C
C  Do the output
C  Write data on files
      call OUTWRITE
C  Write logfile and close data files
      call OUTLOG
      end do ljas
      stop
      end program Li_atom
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCYCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
