C-----------------------------------------------------------------------
      module observables
       use highlevel
       use midlevel
       use random
       use orbital_hf
       use determinant
       use jastrow
       use lattice_Lisolid
C Comprises subroutines for potential, interaction and kinetic energy
C  INTEN  interaction energy (INTENOLD = old value before new draw), a
C         specific electron is considered
C  POTEN  potential energy (in the following OLD appended as above)
C  KINEN  kinetic energy; kinetic energy is also a two-body expression
C         and has to be updated at each step
C  TOTELEN sum of kinetic,potential and interaction energy
C          for specific electron
C  NUNUEN static nucleus-nucleus energy for the whole primitive
C         superlattice cell, contains also constants from other energies
C  AVELEN  average over full QMC run of TOTELEN
C  VARELEN variance of TOTELEN
C  AVALLEL average of TOTELEN over NE electrons, i.e.total
C         energy of all electrons per electron
C  VARALLEL variance of total energy per electron
C  AVBLEL average of total energy as above, intermediately used only
C         for input and control of blocking
C  VARBLEL variance of total energy per electron
C  AVKINEL average of single electron laplace kinetic energy
C  VARKINEL variance of AVKINEL
C  AVVELEL average of single electron velocity kinetic energy
C  VARVELEL variance of AVVELEL
C  AVPOTEL average of single electron potential energy
C  VARPOTEL variance of AVPOTEL
C  AVINTEL average of single electron part of interaction energy
C  VARINTEL variance of AVINTEL
C  AVTOTKIN average of all electrons' kinetic energy per electron
C  VARTOTKIN variance of AVTOTKIN, similarly for the other
C           energy parts, i.e. AVTOTPOT,AVTOTINT
C  AVBLOCKEL block average of single electron total energy
C  VARBLOCKEL block variance of AVBLOCKEL
C  AVALLBLOCKS block average over all electrons
C  VARALLBLOCKS block variance over all electrons
       implicit none
       public :: INITOBS,OBSERV,OBSERVSTATEL,OBSERVSTATALL
       public :: INTENERGY,POTENERGY,KINENERGY,DENSITY
       integer,parameter,public        :: NRHO=1000
       real(dp),public   :: AVTOTALL,AVALLEL,VARTOTALL,DRHO,NUNUEN,
     &       ALLKIN,ALLPOT,ALLINT,AVTOTKIN,VARTOTKIN,
     &       AVTOTPOT,VARTOTPOT,AVTOTINT,VARTOTINT,
     &       ALLBLOCKS,AVALLBLOCKS,VARALLBLOCKS
       real(dp),public,dimension(NE)   :: INTEN,POTEN,KINEN,
     &       POTENOLD,KINENOLD,VELEN,VELENOLD,TOTELEN,
     &       AVELEN,VARELEN,AVBLEL,VARBLEL,AVKINEL,VARKINEL,AVVELEL,
     &       VARVELEL,AVPOTEL,VARPOTEL,AVINTEL,VARINTEL,
     &       AVBLOCKEL,VARBLOCKEL
       real(dp),public,dimension(NRHO,NE) :: RHORAD,RHORADOLD,AVRHORAD
      contains
C-----------------------------------------------------------------------
      subroutine INITOBS
       integer               :: i,ii
       real(dp),dimension(4) :: h
       MCRUN=.false.
       INTEN=EMACH
       POTEN=EMACH
       KINEN=EMACH
       VELEN=EMACH
       TOTELEN=EMACH
       DRHO=0.01
       RHORAD=0.0_dp
C  Calculate the nuclei lattice energy per electron
       NUNUEN = EMACH
       do ii=1,NK
        do i=1,NK
         h(1:3)=(ZK(1:3,i)-ZK(1:3,ii))/dble(N1D)
         h(4) = dsqrt (sum (h(1:3)**2))
         if (h(4) == 0._dp) cycle
C--------------new lattices' division-----------------------------------
C  Different from actual text only the superlattice is used for Ewald.
C  For Ewald with different lattices, bcc and sc (SL) uncomment the
C  following 2 lines and comment the subsequent ones
C        NUNUEN = NUNUEN - (1._dp/h(4) + VEW(SULATTY,h(1:3)) -
C     &           2._dp*PI*h(4)**2/3._dp)
         NUNUEN = NUNUEN + 1._dp/h(4)
C     &         + VEW(SULATTY,h(1:3)) -
C     &           2._dp*PI*h(4)**2/3._dp
C----------end new lattices' division-----------------------------------
        end do
       end do
       NUNUEN = +0.5_dp*CHARGE*NUNUEN/SULATCON/NK
      end subroutine INITOBS
C-----------------------------------------------------------------------
      subroutine INITRANOBS
       MCRUN = .true.
       MCOUNT = 0
       IBLOCKA = 1
       IMCA = 1
       AVELEN=0.0_dp
       AVALLEL=0.0_dp
       VARELEN=0.0_dp
       AVTOTALL=0.0_dp
       VARTOTALL=0.0_dp
       AVKINEL=0.0_dp
       VARKINEL=0.0_dp
       AVBLEL=0.0_dp
       VARBLEL=0.0_dp
       AVBLOCKEL=0.0_dp
       VARBLOCKEL=0.0_dp
       AVALLBLOCKS=0.0_dp
       VARALLBLOCKS=0.0_dp
       AVRHORAD=0.0_dp
      end subroutine INITRANOBS
C-----------------------------------------------------------------------
      subroutine OBSERV
C  The actual random values of the observables are called if the step
C  is accepted, otherwise the old values are taken except the
C  interaction energy which could have changed because positions
C  of other electrons have changed after the last access to this
C  electron.
       call INTENERGY(INTEN)
       call KINENERGY(KINEN,VELEN)
       if(MCSTEP) then
        call POTENERGY(POTEN)
        call DENSITY
       else
        POTEN(IE)=POTENOLD(IE)
        RHORAD(1:NRHO,IE)=RHORADOLD(1:NRHO,IE)
       end if
       TOTELEN(IE)=KINEN(IE)+POTEN(IE)+INTEN(IE)
      end subroutine OBSERV
C-----------------------------------------------------------------------
      subroutine OBSERVSTATEL
C  The statistics for the observables is calculated for each electron
C  separately. The new values are named as old ones for the start of a
C  new IMC step.
         POTENOLD(IE)=POTEN(IE)
         RHORADOLD(1:NRHO,IE)=RHORAD(1:NRHO,IE)
         call AVVAR(IMC,TOTELEN(IE),AVELEN(IE),VARELEN(IE))
         call AVVAR(IMC,KINEN(IE),AVKINEL(IE),VARKINEL(IE))
         call AVVAR(IMC,VELEN(IE),AVVELEL(IE),VARVELEL(IE))
         call AVVAR(IMC,POTEN(IE),AVPOTEL(IE),VARPOTEL(IE))
         call AVVAR(IMC,INTEN(IE),AVINTEL(IE),VARINTEL(IE))
         AVRHORAD(1:NRHO,IE)=AVRHORAD(1:NRHO,IE)*dble(IMC)/dble(IMC+1)+
     &              RHORAD(1:NRHO,IE)/dble(IMC+1)
         call BLOCKING(TOTELEN,AVBLEL,VARBLEL,AVBLOCKEL,VARBLOCKEL)
      end subroutine OBSERVSTATEL
C-----------------------------------------------------------------------
      subroutine OBSERVSTATALL
C  Calculates the overall per electron statistics, i.e. for the
C  average of NE electrons.
       AVALLEL = sum (TOTELEN(1:NE))/dble(NE)
       call AVVAR(IMC,AVALLEL,AVTOTALL,VARTOTALL)
       ALLKIN = sum (KINEN(1:NE))/dble(NE)
       call AVVAR(IMC,ALLKIN,AVTOTKIN,VARTOTKIN)
       ALLPOT = sum (POTEN(1:NE))/dble(NE)
       call AVVAR(IMC,ALLPOT,AVTOTPOT,VARTOTPOT)
       ALLINT = sum (INTEN(1:NE))/dble(NE)
       call AVVAR(IMC,ALLINT,AVTOTINT,VARTOTINT)
       ALLBLOCKS = sum (AVBLOCKEL(1:NE))/dble(NE)
       call AVVAR(IMC,ALLBLOCKS,AVALLBLOCKS,VARALLBLOCKS)
      end subroutine OBSERVSTATALL
C-----------------------------------------------------------------------
      subroutine INTENERGY(inten)
C  Calculates interaction energy. The factor 0.5 at the Coulomb double
C  sum is taken into account here, inten(IE)=0.5*\sum_k 1/|r_IE - r_k| +
C  VEW(LA,r_IE - r_k) with table VEW(.,.). This table is not
C  identical to the Ewald potential, but has added the homogeneous
C  charge with potenial =(+ 2*PI*r^2/(3*SULATCON^3)) (which makes it
C  smoothly continuous at the boundary). It has to be taken out
C  because it does not belong to the real interaction energy. It is
C  kept only in the Jastrow factor. Observe that VEW uses coordinates
C  within the Wigner-Seitz cell in units of SULATCON.
C
C  inten(IE)= contribution to the interaction energy of IE-th electron
       real(dp),intent(out),dimension(NE) :: inten
C
       integer :: i,k
       real(dp) :: hw
       real(dp),dimension(4) ::hrj
       hw=0.0_dp
C  Here the electron-electron distances are updated
       call RDIST(RE,RENEW)
C  Coulomb el-el interaction is summed
       ielekpot:do k=1,NE
        if (k == IE) then
         cycle ielekpot
        end if
        if (MCSTEP) then
         hrj(1:4) = DISTNEW(1:4,IE,k)/SULATCON
        else
         hrj(1:4) = DIST(1:4,IE,k)/SULATCON
        end if
C  Take care that the table VEW does not contain the harmonic potential
C  and subtract it.
        hw = hw + 1.0_dp/hrj(4)
C     &    + VEW(SULATTY,hrj(1:3)) -
C     &      2._dp*PI*hrj(4)**2/3._dp
       end do ielekpot
       inten(IE)=0.5_dp*hw/SULATCON
      end subroutine INTENERGY
C-----------------------------------------------------------------------
      subroutine POTENERGY(poten)
C  Calculates one-particle potential energy. Here the bcc lattice of
C  the nuclei is used. The nucleus which is closest to the electron IE
C  is chosen as the origin of the primitive bcc cell. The rest comes
C  via Ewald. Take care of that the table VEW(.,.) has implicitly
C  added the harmonic potential and should be subtracted again.
C  The energy is composed of the  centrosymmetric potentials of all
C  nuclei.
       real(dp),intent(out),dimension(NE)  :: poten
C
       integer  ::  ik
       real(dp)  ::  r,h
       real(dp),dimension(3,NK) :: er,ez
C  Here the distances between electron IE and the nuclei are updated !already done in main routine
C       do ik=1,NK
C        er(1:3,ik) = RENEW(1:3,IE) - RK(1:3,ik)
C        ez(1:3,ik) = er(1:3,ik)/SULATCON
C        call FOLDTOWSC(ez(1:3,ik))
C        er(1:3,ik) = ez(1:3,ik)*SULATCON
C       end do
C       RKE(1:3,1:NK,IE) = er(1:3,1:NK)
C       ZKE(1:3,1:NK,IE) = RKE(1:3,1:NK,IE)/LATCON
C       call MINDIST(NK,NE,RKE,IKM)
C--------------new lattices' division-----------------------------------
C  Different from actual text only the superlattice is used for Ewald.
C  For Ewald with different lattices, bcc and sc (SL) uncomment the
C  following 5 lines and comment the subsequent ones
C       r = dsqrt (sum (ZKENEW(1:3,IKM(IE),IE)**2))
C       r = max (r,JASEMACH)
C      poten(IE) = -(CHARGE/r/LATCON +
C     &              CHARGE*(VEW(LATTY,ZKENEW(1:3,IKM(IE),IE)) -
C     &              2._dp*PI*r**2/3._dp)/LATCON)
       h=0.0_dp
       do ik =1,NK
        r = RKENEW(4,ik,IE)/SULATCON
        r = max (r,JASEMACH)
        h = h + 1/r
C     &        + VEW(SULATTY,RKENEW(1:3,ik,IE)/SULATCON) -
C     &        2._dp*PI*r**2/3._dp
       end do
       poten(IE) = -h*CHARGE/SULATCON
C----------end new lattices' division-----------------------------------
      end subroutine POTENERGY
C-----------------------------------------------------------------------
      subroutine KINENERGY(kinen,velen)
C  Calculates kinetic energy kinen(IE) of electron IE.
C  It includes the kinetic energy originating from the determinant
C  and the Jastrow factor. The latter is a two-body term which
C  is completed by the Ewald potential to reflect the periodic system
C  completion. Here, in the Jastrow, factor we use the smoothly
C  continuous potential to get rid of artificial fluctuations. We are
C  allowed to do so because it is just an Ansatz for optimizing
C  the wavefunction.
       real(dp),intent(out),dimension(NE)    :: kinen,velen
C
       integer                               :: iii
       real(dp)                              :: hkin,qsel
       real(dp)                              :: hlad,hlaj
       real(dp),dimension(3)                 :: hgrd,hgrj
       real(dp),dimension(3,NK,NORB)         :: pgrnew
       real(dp),dimension(NK,NORB)           :: planew
       if (MCSTEP) then
        qsel = QD(IES)
        call ORBDER(RKENEW(1:4,1:NK,IE),pgrnew,planew)
        hgrj(1:3)=GRJAS(1:3,IEES,IES)
        hlaj=LAPJAS(IEES,IES)
       else
        qsel = 1._dp
        call ORBDER(RKE(1:4,1:NK,IE),pgrnew,planew)
        hgrj(1:3)=GRJASOLD(1:3,IEES,IES)
        hlaj=LAPJASOLD(IEES,IES)
       end if
       call SLAKIN(AOLD(1:IENS,1:IENS,IES),
     &        pgrnew,planew,GRADDET(1:3,IEES,IES),LAPLDET(IEES,IES))
       hgrd(1:3)=GRADDET(1:3,IEES,IES)/qsel
       hlad=LAPLDET(IEES,IES)/qsel
       hkin = hlad+hlaj+2.0_dp*dot_product(hgrd,hgrj)
       velen(IE) = 0.5_dp*sum ((hgrd(1:3)+hgrj(1:3))**2)
       kinen(IE) = -0.5_dp*hkin
C       write(*,*)'IE,kinen=',IE,kinen(IE)
      end subroutine KINENERGY
C-----------------------------------------------------------------------
      subroutine DENSITY
C   Radial density RHORAD is a function of distance s from nucleus and
C   normalized to 1=sum_s 4*PI*s**2 ds RHORAD(s). It is discretized
C   in units of DRHO with NRHO sections. Values below DRHO are added
C   to first unit and those above NRHO*DRHO added to last unit.
       integer    :: j
       real(dp)   :: s,h
       RHORAD=0.0_dp
       h=4.0_dp*PI*DRHO
       s=max (sqrt(sum (RKENEW(1:3,IKM(IE),IE)**2)),DRHO+EMACH)
       s=min (s,NRHO*DRHO)
C       if (s > LATCON) write(*,*)'s=',s
       j=int (s/DRHO)
       RHORAD(j,IE)=1/h/s**2
      end subroutine DENSITY
C-----------------------------------------------------------------------
      end module observables
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
