C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C   autcorrelation function still under construction, also within INITRAN
      module random
       use highlevel
       use midlevel
C  controls the sequence of random numbers,
C  allows for use of different random generators
C  MAKF number of step positions for autocorrelation function
       implicit none
       public :: INITRAN,INITRANNUMB,FILLRAN,FILLG95,FILLTAO,RANF,
     &           AUTOCORR,AVVAR,BLOCKING
       integer,parameter,public         :: MRAN=10001,ISEED=1820459103
       integer,parameter,public         :: MAKF=40,MTAKF=2000
       integer,public                   :: IRAN,IFRAN,ISEED0,ISEED1
       integer,public                   :: TMAX,TFMAX,TP
       integer,dimension(MTAKF),public  :: TAU
       real,dimension(MRAN),public      :: FRAN
       real(dp),dimension(MAKF),public  :: FTF,FTA
       real(dp),dimension(MTAKF),public :: AKFF,FTAU
       character(40), public            :: RANDNAME
      contains
C-----------------------------------------------------------------------
      subroutine INITRAN
       integer                   :: ia
       write(*,*)'MCPRE,MCMAX,MAXA=?'
       read(*,*)MCPRE,MCMAX,MAXA
C       MCPRE=100
C       MCMAX=1000
C       MAXA=100
       RANDNAME="random generator from TAO               "
       ISEED0 = ISEED
       CALL FILLRAN
C  Read the abscissa integers for the autocorrelation function, data
C  scale enlarges with distance from zero
       open(unit=40,file='AKF.DAT',status='old')
       TMAX = 0
       do ia=1,MAKF
        read (40,*,end=999) TAU(ia)
        if (TMAX < tau(ia)) TMAX=TAU(ia)
        TFMAX = ia
       end do
       close(40)
999    write(*,*)'AKF: number of abscissa values read-in, TFMAX= ',
     &  TFMAX,' should be equal to MAKF= ',MAKF
       AKFF = 0
      end subroutine INITRAN
C-----------------------------------------------------------------------
      subroutine INITRANNUMB
C  Set the initial electron positions at random
       integer   :: j,jj
       real(dp)  :: rd
       do j=1,NE
        do jj=1,3
         rd = FRAN(IFRAN)-0.5
         RE(jj,j) = rd
         RENEW(jj,j) = rd
         IFRAN = IFRAN + 1
         if (IFRAN > MRAN) then
          IRAN = IRAN + MRAN
          call FILLRAN
         end if
        end do
       end do
      end subroutine INITRANNUMB
C-----------------------------------------------------------------------
       subroutine FILLRAN()
        select case (RANDNAME)
         case("random generator from g95               ")
           call FILLG95()
         case("random generator from TAO               ")
           call FILLTAO()
         case default
           call FILLG95()
        end select
       end subroutine FILLRAN
C
       subroutine FILLG95()
C        integer :: i
C        ISEED1 = ISEED
C        do i=1,MRAN
CC  below the g95 random generator which seems ok
C          FRAN(i) = rand(IRAN+i)
C        end do
C        IFRAN = 1
       end subroutine FILLG95
C-----------------------------------------------------------------------
       subroutine FILLTAO()
        integer :: i
        ISEED1 = ISEED0
        do i=1,MRAN
C  below from the book, "An Introduction to Computational Physics"
C  written by Tao Pang and published and copyrighted
C  by Cambridge University Press in 1997
          FRAN(i)  = RANF()
        end do
        ISEED0 = ISEED1
        IFRAN = 1
       end subroutine FILLTAO
C
       subroutine AUTOCORR()
        real :: AV=0.0,VA=0.0
        real,dimension(MRAN) :: CORR,AVER,VARI
        integer :: i,k,n
        IRAN = 1
        IFRAN =1
        call FILLG95()
        IRAN = IRAN + MRAN
        do k=1,MRAN-1
          CORR(K) = 0.0
          AV = FRAN(1)
          do i=2,MRAN
            AV = AV*real(i-1)/real(i) + FRAN(i)/real(i)
            VA = VA*real(i-1)/real(i) + (AV-FRAN(i))**2/real(i-1)
            n = modulo(i+k,MRAN)
            CORR(k)=CORR(k)+FRAN(i)*FRAN(n)
          end do
          AVER(k) = AV
          VARI(k) = VA
          CORR(k)=(CORR(k)/real(MRAN)-AV**2)/VA
        end do
        do k=1,MRAN-1
          write(50,*) k,CORR(k),'AV= ',AVER(k),'VA= ',VARI(k)
        end do
       end subroutine AUTOCORR
C-----------------------------------------------------------------------
      function RANF() result(Z)
        real :: Z
        integer,parameter :: ia=16807,ic=2147483647,iq=127773,ir=2836
        integer :: ih,il,it
C  IC = 2**31-1 exists for 32 bit chips
        ih = ISEED1/iq
        il = MOD(ISEED1,iq)
        it = ia*il-ir*ih
        if (it.GT.0) then
          ISEED1 = it
        else
          ISEED1 = ic+it
        end if
        Z = ISEED1/FLOAT(ic)
      end function RANF
C-----------------------------------------------------------------------
      subroutine AVVAR(i,f,avgf,varf)
       integer,intent(in)       :: i
       real(dp),intent(in)      :: f
       real(dp),intent(inout)   :: avgf,varf
       if (i == 1) then
        avgf=f
        varf=0.0_dp
       else
        avgf=avgf+(f-avgf)/dble(i)
        varf=dble(i-2)/dble(i-1)*varf+
     &       dble(i)/dble(i-1)**2*(avgf-f)**2
       end if
      end subroutine AVVAR
C-----------------------------------------------------------------------
      subroutine BLOCKING(fa,avgfa,varfa,avgblocka,varblocka)
       real(dp),intent(in),dimension(NE)   :: fa
       real(dp),intent(inout),dimension(NE)  :: avgfa,varfa,avgblocka,
     &                                          varblocka
C  Blocks of length MAXA, where index IMCA counts within a block and
C  index IBLOCKA counts the MCMAX/MAXA blocks.
C  Input is the random variable fa and output is average and variance:
C  at each Monte-Carlo step avgfa,varfa and
C  after each block avblocka, varblocka.
C  The blocking is used for every electron separately.
       if (IBLOCKA > MCMAX/MAXA) write(*,*) 'Error BLOCKING:
     &    Step IMC=',IMC,' goes beyond last block IBLOCKA=',IBLOCKA,
     &    'which should be MCMAX/MAXA=',MCMAX/MAXA
       if (IMCA == 1) then
        avgfa(IE) = fa(IE)
        varfa(IE) = 0.0_dp
       else
        avgfa(IE)=avgfa(IE)+(fa(IE)-avgfa(IE))/dble(IMCA)
        varfa(IE)=dble(IMCA-2)/dble(IMCA-1)*varfa(IE)+
     &       dble(IMCA)/dble(IMCA-1)**2*(avgfa(IE)-fa(IE))**2
       end if
       if (IE == NE) then
        IMCA=IMCA+1
        if (IMCA == MAXA+1) then
C  Collect the blocking intermediate results
         if (IBLOCKA == 1) then
          avgblocka=avgfa
          varblocka=0.0_dp
         else
          avgblocka=avgblocka+
     &               (avgfa-avgblocka)/dble(IBLOCKA)
          varblocka=dble(IBLOCKA-2)/dble(IBLOCKA-1)*varblocka+
     &    dble(IBLOCKA)/dble(IBLOCKA-1)**2*(avgblocka-avgfa)**2
         end if
         IBLOCKA = IBLOCKA + 1
         IMCA = 1
        end if
       end if
      end subroutine BLOCKING
C-----------------------------------------------------------------------
      subroutine AKF(ff,ffav)
       real(dp),intent(in)                 :: ff,ffav
C  Calculates the autocorrelation function of variable ff
C  dependent on the number of MC steps t between two times. The maximum
C  time TMAX should be much lesser than the total length of the MC run,
C  e.g. to be something as the block length.
C  ff= actually realized value of the random variable
C  ffav= actual average value of ff up to this step and including it.
C
C  A ring memory is associated with the MC steps, i.e. after the
C  number TMAX of elapsed steps the subsequent steps start again at 1.
       integer                             :: ia,tb
       real(dp)                            :: a,b,difa,difb,f,fa
       f = ff
       fa = ffav
       if (IMC > 1) then
        b = 1.0_dp/dble(IMC-1)
        difb = f - fa
        TP = TP + 1
        if (TP > TMAX) TP=TP-TMAX
        do ia=1,TFMAX
         if (TAU(ia) > (IMC-2)) then
          if (TAU(ia) == IMC) FTAU(ia)=fa
          cycle
         end if
         a = 1.0_dp/dble(IMC-TAU(ia)-1)
         tb = TP - TAU(ia)
         if (tb < 1) tb=tb+TMAX
         difa = FTF(tb) - FTA(tb)
         AKFF(ia) = AKFF(ia)*(1.0_dp-a) + difb*a*b*(dble(IMC)*difa+
     &              dble(TAU(ia))*(FTA(tb)+FTAU(ia)-2.0_dp*fa+difb*b))
        end do
       else
        TP = IMC
        if (TAU(1) == 1) FTAU(1)=fa
       end if
       FTF(TP) = f
       FTA(TP) = fa
      end subroutine AKF
C-----------------------------------------------------------------------
      end module random
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
