CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCYCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      program jellium_cluster
C
C  QMC main program of spherical jellium cluster
C

       use highlevel
       use midlevel
       use orbital
       use determinant
       use random
       use jastrow, JEXP=>JASEXPATOM
       use observables
       use output

       implicit none
       integer          :: i,ii,hi,j,jj,islap
       real(dp)         :: q,rd

C  Open output data files
C  Contained in M_output.f
       call INITOUT

C  Initialize the parameters for the Monte Carlo run.
C  Fill an array with a set of random numbers to be available.
C  Contained in M_random.f.
       call INITRAN

C  Set the initial electron positions at random.
C  Contained in M_random.f
       call INITRANNUMB

C  Associate electron with single particle wavefunction,
C  and determine initial wavefunction matrix;
C  Contained in M_orbital_cluster.f
       call INITORB

C  Set initial values for Jastrow parameters
C  Contained in M_jastrow.f
       call INITJAS

C  Set initial values for inverse matrices;
C  contained in M_determinant.f
       call INITDET

C  Set the initial values of the MC run;
C  contained in M_observables.f
       call INITOBS
C
C  MC loop:
C  First, the prerun for thermalizing
      lrun:do IMCR=1,MCMAX
C  Start the main run with statistical sampling of the observables if
C  the number of prerun steps is reached. Initialize the observables
C  and their statistics
       if ( IMCR == MCPRE) then
        call INITRANOBS
       end if
       IMC=IMCR-MCPRE+1
C
C  Inner loop on electron index:
       lelrun:do IE=1,NE
        IES=1
        if (IE > NES(1)) IES=2
        hi=(IES-1)*NES(1)
        IEES=IE-hi
        IENS=NES(IES)
        do i=1,3
C  Shift position at random within +-STEPMAX/2
         rd = (FRAN(IFRAN)-0.5)*STEPMAX
         IFRAN = IFRAN + 1
         if (IFRAN > MRAN) then
          IRAN = IRAN + MRAN
          call FILLRAN
         end if
         RENEW(i,IE) = RE(i,IE)+rd
        end do
C  Calculate Jastrow factor quantities
        call JEXP
C  Calculate single particle wavefunction part
        call ORBWAV(RENEW(1:3,IE),PSINEW)
        do ii=1,IENS
         PSIMAT(ii,IEES,IES)=PSINEW(NELORB(hi+ii))
        end do
        call SLAQUOT(AOLD(1:IENS,1:IENS,IES),
     &               PSIMAT(1:IENS,1:IENS,IES),QD(IES))
C  Test on acceptance
        q = (QD(IES)*QJ(IES))**2
        if (q < 1) then
         MCSTEP = (dble(FRAN(IFRAN)) < q)
         IFRAN = IFRAN + 1
         if (IFRAN .gt. MRAN) then
          IRAN=IRAN+MRAN
          call FILLRAN
         end if
        else
         MCSTEP = .true.
        end if
C  Calculates observables
        if (MCRUN) then
         call OBSERV
        end if
C  Update for acceptance or not
        lmcstep:if (MCSTEP) then
         RE(1:3,IE) = RENEW(1:3,IE)
         call SLASM(QD(IES),AOLD(1:IENS,1:IENS,IES),
     &         PSIMAT(1:IENS,1:IENS,IES),ANEW(1:IENS,1:IENS,IES))
         AOLD(1:IENS,1:IENS,IES) = ANEW(1:IENS,1:IENS,IES)
         MCOUNT = MCOUNT + 1
        end if lmcstep
C  Calculate statistical results of observables per electron
        if (MCRUN) call OBSERVSTATEL
       end do lelrun
C
C  Calculate statistical results of observables for all electrons
       if (MCRUN) call OBSERVSTATALL
      end do lrun
C
C  Do the output
C  Write data on files
      call OUTWRITE
C  Write logfile and close data files
      call OUTLOG

      stop
      end program jellium_cluster
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
