CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  A MC run for the thermodynamical energy, a.u. used
C
C
C-----------------------------------------------------------------------
      module random
C  controls the sequence of random numbers,
C  allows for use of different random generators
       implicit none
       public :: FILLG95,FILLTAO,RANF,AUTOCORR,GENRAN
       integer,parameter,public ::
     &   dp=selected_real_kind(2*precision(1.0))
       integer      :: MCMAX,MCPRE,MCOUNT
       integer,parameter,public :: MRAN=10001,ISEED=1820459103
       integer,parameter,public :: MAKF=10000
       integer,public :: IRAN,IFRAN,ISEED0,ISEED1,SEED
       real,dimension(MRAN),public :: FRAN
       character(40) :: RANDNAME
       DOUBLE PRECISION,private :: B=4.294967291D9,C=0.0D0
       DOUBLE PRECISION,dimension(43) :: ZBQLIX
       data  ZBQLIX /8.001441D7,5.5321801D8,
     &  1.69570999D8,2.88589940D8,2.91581871D8,1.03842493D8,
     &  7.9952507D7,3.81202335D8,3.11575334D8,4.02878631D8,
     &  2.49757109D8,1.15192595D8,2.10629619D8,3.99952890D8,
     &  4.12280521D8,1.33873288D8,7.1345525D7,2.23467704D8,
     &  2.82934796D8,9.9756750D7,1.68564303D8,2.86817366D8,
     &  1.14310713D8,3.47045253D8,9.3762426D7 ,1.09670477D8,
     &  3.20029657D8,3.26369301D8,9.441177D6,3.53244738D8,
     &  2.44771580D8,1.59804337D8,2.07319904D8,3.37342907D8,
     &  3.75423178D8,7.0893571D7 ,4.26059785D8,3.95854390D8,
     &  2.0081010D7,5.9250059D7,1.62176640D8,3.20429173D8,
     &  2.63576576D8/
      contains
C
C-----------------------------------------------------------------------
      subroutine INITRAN
       integer                     :: ia
       real(dp)                    :: aver,vari
       real(dp),dimension(MAKF)    :: corr
       real(dp),dimension(MCMAX)   :: rn
C       write(*,*)'MCPRE,MCMAX,MAXA=?'
C       read(*,*)MCPRE,MCMAX,MAXA
C       MCPRE=100
C       MCMAX=1000
C       MAXA=100
C       RANDNAME="random generator from REC_PJN           "
C       RANDNAME="random generator from TAO               "
C       RANDNAME="random generator from G95               "
       RANDNAME="random generator from F90/95            "
       select case (RANDNAME)
         case("random generator from TAO               ")
           write(35,*)'random generator from TAO               '
           ISEED0 = ISEED
           call FILLTAO
         case("random generator from G95               ")
           write(35,*)'random generator from G95               '
           call srand(ISEED)
           call FILLG95
         case("random generator from REC_PJN           ")
           write(35,*)'random generator from REC_PJN           '
           SEED = 11 ! or some other integer
           call ZBQLINI(SEED)
         case("random generator from F90/95            ")
           write(35,*)'random generator from F90/95            '
C  no further initialization for repeatable runs
         case default
           write(35,*)'No random generator! Stop!'
           stop
        end select
C  Test on autocorrelation
C        do ia = 1,MCMAX
C          call GENRAN(rn(ia))
C        end do
C        call AUTOCORR(MCMAX,MAKF,rn,aver,vari,corr)
C        do ia = 1,MAKF
C          write (30,*)ia,corr(ia)
C        end do
      end subroutine INITRAN
C-----------------------------------------------------------------------
       subroutine GENRAN(rannumb)
        DOUBLE PRECISION,intent(out) :: rannumb
C
        DOUBLE PRECISION :: dum
C
        select case (RANDNAME)
         case("random generator from TAO               ")
           rannumb = FRAN(IFRAN)
           IFRAN = IFRAN + 1
           if (IFRAN .gt. MRAN) THEN
             IRAN=IRAN+MRAN
             call FILLTAO
           end if
         case("random generator from G95               ")
           rannumb = FRAN(IFRAN)
           IFRAN = IFRAN + 1
           if (IFRAN .gt. MRAN) THEN
             IRAN=IRAN+MRAN
             call FILLG95
           end if
         case("random generator from REC_PJN           ")
           rannumb = ZBQLU01(dum)
         case("random generator from F90/95            ")
           call random_number(rannumb)
         case default
           write(*,*)'No random generator!'
           stop
        end select
       end subroutine GENRAN
C-----------------------------------------------------------------------
       subroutine FILLG95()
        integer :: i
        do i=1,MRAN
C  below the g95 random generator which does not seem ok
        FRAN(i) = rand()
        end do
        IFRAN = 1
       end subroutine FILLG95
C-----------------------------------------------------------------------
       subroutine FILLTAO()
        integer :: i
        ISEED1 = ISEED0
        do i=1,MRAN
C  below from the book, "An Introduction to Computational Physics"
C  written by Tao Pang and published and copyrighted
C  by Cambridge University Press in 1997
          FRAN(i)  = RANF()
        end do
        ISEED0 = ISEED1
        IFRAN = 1
       end subroutine FILLTAO
C-----------------------------------------------------------------------
       subroutine AUTOCORR(mc,mx,x,aver,vari,corr)
C   mc = no of MC steps
C   mx = no of points for autocorrelation function
        integer,intent(in)                    :: mc,mx
        real(dp),dimension(mx),intent(in)     :: x
        real(dp),intent(out)                  :: aver,vari
        real(dp),dimension(mx), intent(out) :: corr
        integer :: i,k,n
        aver = x(1)
        do n=2,mc
         vari = ((n-2)*vari)/(n-1) + (x(n)-aver)**2/n
         aver = ((n-1)*aver)/n + x(n)/n
        end do
C   correct variance for small number of events
        vari = (vari*mc)/(mc-1)
        write (35,*)'hello from AUTO'
        corr = 0._dp
         do k=1,mx-1
          do i=1,mc-k
            corr(k) = corr(k) + (x(i)-aver)*(x(i+k)-aver)
          end do
          corr(k) = corr(k)/(vari*(mc-1))
         end do
C        write (50,*) 'no. events mx = ',mx,'  average = ', aver,
C     &      '  variance =',vari
C        do k=1,mx-1
C          write (50,*) k,corr(k)
C        end do
       end subroutine AUTOCORR
C-----------------------------------------------------------------------
       function RANF() result(Z)
        real :: Z
        integer,parameter :: ia=16807,ic=2147483647,iq=127773,ir=2836
        integer :: ih,il,it
C  IC = 2**31-1 exists for 32 bit chips
        ih = ISEED1/iq
        il = MOD(ISEED1,iq)
        it = ia*il-ir*ih
        if (it.GT.0) then
          ISEED1 = it
        else
          ISEED1 = ic+it
        end if
        Z = ISEED1/FLOAT(ic)
       end function RANF
C
C-----------------------------------------------------------------------
C      end module random
C***************below from***********************************************
C********	AUTHORS: Richard Chandler		***********
C********		 (richard@stats.ucl.ac.uk)	***********
C********		 Paul Northrop 			***********
C********		 (northrop@stats.ox.ac.uk)	***********
C********	LAST MODIFIED: 26/8/03			***********
C*******************************************************************
C      module random_rec_pjn
C
C       Initializes seed array etc. for random number generator.
C       The values below have themselves been generated using the
C       NAG generator. The initialization and
C       a subroutine for uniform random numbers in (0,1] is
C       contained.
C
C      integer :: SEED
C      DOUBLE PRECISION :: B=4.294967291D9,C=0.0D0
C      DOUBLE PRECISION,dimension(43) :: ZBQLIX
C      data  ZBQLIX /8.001441D7,5.5321801D8,
c     &  1.69570999D8,2.88589940D8,2.91581871D8,1.03842493D8,
C     &  7.9952507D7,3.81202335D8,3.11575334D8,4.02878631D8,
C     &  2.49757109D8,1.15192595D8,2.10629619D8,3.99952890D8,
C     &  4.12280521D8,1.33873288D8,7.1345525D7,2.23467704D8,
C     &  2.82934796D8,9.9756750D7,1.68564303D8,2.86817366D8,
C     &  1.14310713D8,3.47045253D8,9.3762426D7 ,1.09670477D8,
C     &  3.20029657D8,3.26369301D8,9.441177D6,3.53244738D8,
C     &  2.44771580D8,1.59804337D8,2.07319904D8,3.37342907D8,
C     &  3.75423178D8,7.0893571D7 ,4.26059785D8,3.95854390D8,
C     &  2.0081010D7,5.9250059D7,1.62176640D8,3.20429173D8,
C     &  2.63576576D8/
C
C
C     contains
C******************************************************************
      SUBROUTINE ZBQLINI(SEED)
C       To initialize the random number generator - either
C       repeatably or nonrepeatably. Need double precision
C       variables because integer storage can't handle the
C       numbers involved
C	SEED	(integer, input). User-input number which generates
C		elements of the array ZBQLIX, which is subsequently used
C		in the random number generation algorithm. If SEED=0,
C		the array is seeded using the system clock if the
C		FORTRAN implementation allows it. This feature for
C       nonrepeatable runs has been cancelled here.
C	LFLNO	(integer). Effect is obsolete here.
C       Number of lowest file handle to try when
C		opening a temporary file to copy the system clock into.
C		Default is 80 to keep out of the way of any existing
C		open files (although the program keeps searching till
C		it finds an available handle). If this causes problems,
C               (which will only happen if handles 80 through 99 are
C               already in use), decrease the default value.
C******************************************************************
      INTEGER LFLNO
      PARAMETER (LFLNO=80)
C******************************************************************
C	ZBQLIX	Seed array for the random number generator. Defined
C		in ZBQLBD01
C	B,C	Used in congruential initialisation of ZBQLIX
C	FILNO	File handle used for temporary file
C	INIT	Indicates whether generator has already been initialised
C
      INTEGER SEED,SS,MM,HH,DD,FILNO,I
      DOUBLE PRECISION TMPVAR1,DSS,DMM,DHH,DDD
C
      IF (SEED.EQ.0) THEN
       write(*,*)'SEED=0 not allowed in this package'
       stop
      ELSE
       TMPVAR1 = DMOD(DBLE(SEED),B)
      ENDIF
      ZBQLIX(1) = TMPVAR1
      DO 100 I = 2,43
       TMPVAR1 = ZBQLIX(I-1)*3.0269D4
       TMPVAR1 = DMOD(TMPVAR1,B)
       ZBQLIX(I) = TMPVAR1
 100  CONTINUE

 1    FORMAT(//5X,'****WARNING**** You have called routine ZBQLINI ',
     +'more than',/5X,'once. I''m ignoring any subsequent calls.',//)
 2    FORMAT(//5X,'**** ERROR **** In routine ZBQLINI, I couldn''t',
     +' find an',/5X,
     +'available file number. To rectify the problem, decrease the ',
     +'value of',/5X,
     +'the parameter LFLNO at the start of this routine (in file ',
     +'random_rec_pjn.f)',/5X,
     +'and recompile. Any number less than 100 should work.')
C
      end subroutine ZBQLINI
C*****************************************************************
      FUNCTION ZBQLU01(DUMMY)
C
C       Returns a uniform random number between 0 & 1, using
C       a Marsaglia-Zaman type subtract-with-borrow generator.
C       Uses double precision, rather than integer, arithmetic
C       throughout because MZ's integer constants overflow
C       32-bit integer storage (which goes from -2^31 to 2^31).
C       Ideally, we would explicitly truncate all integer
C       quantities at each stage to ensure that the double
C       precision representations do not accumulate approximation
C       error; however, on some machines the use of DNINT to
C       accomplish this is *seriously* slow (run-time increased
C       by a factor of about 3). This double precision version
C       has been tested against an integer implementation that
C       uses long integers (non-standard and, again, slow) -
C       the output was identical up to the 16th decimal place
C       after 10^10 calls, so we're probably OK ...
C
      DOUBLE PRECISION ZBQLU01,DUMMY,X,B2,BINV
      INTEGER CURPOS,ID22,ID43

      SAVE CURPOS,ID22,ID43
      DATA CURPOS,ID22,ID43 /1,22,43/

      B2 = B
      BINV = 1.0D0/B
 5    X = ZBQLIX(ID22) - ZBQLIX(ID43) - C
      IF (X.LT.0.0D0) THEN
       X = X + B
       C = 1.0D0
      ELSE
       C = 0.0D0
      ENDIF
      ZBQLIX(ID43) = X
C
C     Update array pointers. Do explicit check for bounds of each to
C     avoid expense of modular arithmetic. If one of them is 0 the others
C     won't be
C
      CURPOS = CURPOS - 1
      ID22 = ID22 - 1
      ID43 = ID43 - 1
      IF (CURPOS.EQ.0) THEN
       CURPOS=43
      ELSEIF (ID22.EQ.0) THEN
       ID22 = 43
      ELSEIF (ID43.EQ.0) THEN
       ID43 = 43
      ENDIF
C
C     The integer arithmetic there can yield X=0, which can cause
C     problems in subsequent routines (e.g. ZBQLEXP). The problem
C     is simply that X is discrete whereas U is supposed to
C     be continuous - hence if X is 0, go back and generate another
C     X and return X/B^2 (etc.), which will be uniform on (0,1/B).
C
      IF (X.LT.BINV) THEN
       B2 = B2*B
       GOTO 5
      ENDIF
C
      ZBQLU01 = X/B2
C
      end function ZBQLU01
C-----------------------------------------------------------------------
C      end module random_rec_pjn
       end module random
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      module position
C  updates positions, their differences and two-particle potential and
C  its change
       use random
       implicit none
       public :: JASEXP,DENSITY,DENSITY1
       integer,parameter,public :: NEMAX=100,NDIVMX=101
       integer,public :: NE,IE,NDIV,IMC
       real(kind=dp),parameter,public:: EMACH=1.0d-4
       real(kind=dp),public:: LENGTH
       real(kind=dp),dimension(NEMAX),public:: VNEW,VDIF
       real(kind=dp),dimension(3,NEMAX),public :: RE,RNEU
       real(kind=dp),dimension(4,NEMAX,NEMAX),public :: DIST,DISTNEU
       real(kind=dp),dimension(NDIVMX,NDIVMX,NDIVMX) :: RHO
      contains
C-----------------------------------------------------------------------
       subroutine JASEXP(vn,vd)
C  updates the distances from the active electron to all others
        real(kind=dp),intent(out):: vn,vd
        integer :: i,k,n
        real(dp) :: work01,work02,work1,work2
        work1=0.0_dp
        work2=0.0_dp
        IENEK:do k=1,NE
         if (k .eq. IE) then
          cycle
         end if
         DIST(1:3,IE,k)=RE(1:3,IE)-RE(1:3,k)
         DISTNEU(1:3,IE,k)=RNEU(1:3,IE)-RE(1:3,k)
         work01=dsqrt(sum (DIST(1:3,IE,k)**2))
         work02=dsqrt(sum (DISTNEU(1:3,IE,k)**2))
         if (work01 .lt. EMACH)  work01 = 2.D0/3.D0*EMACH
         if (work02 .lt. EMACH)  work02 = 2.D0/3.D0*EMACH
         DIST(4,IE,k) = work01
         DISTNEU(4,IE,k) = work02
         work1=work1+1.D0/work02*dexp(-work02)-1.D0/work01*dexp(-work01)
         work2=work2+1.D0/work02*dexp(-work02)
        end do IENEK
        vd=work1
        vn=work2
       end subroutine JASEXP
C-----------------------------------------------------------------------
       subroutine DENSITY(rh)
C  calculate the average particle density rh() on a cubic mesh
C  with NDIV intervals on each cubic axis, NDIV must be odd
        real(dp),intent(out),dimension(NDIVMX,NDIVMX,NDIVMX) :: rh
        integer :: nx,ny,nz,i
        real(dp) :: dl
        if (dble((NDIV-1)/2) .ne. dble(NDIV-1)/2.0_dp) then
         write(*,*) 'NDIV not odd: stop'
         stop
        end if
        rh = 0.0_dp
        dl = LENGTH/dble(NDIV-1)
        do i=1,NE
         nx = 1 + (NDIV-1)/2 + int(RE(1,i)/dl)
         ny = 1 + (NDIV-1)/2 + int(RE(2,i)/dl)
         nz = 1 + (NDIV-1)/2 + int(RE(3,i)/dl)
         if ((nx > NDIV) .or. (ny > NDIV) .or.
     .      (nz > NDIV)) then
          write(*,*)'too large nx, ny, or nz ',nx,ny,nz,' > NDIV ',NDIV
          stop
         end if
         rh(nx,ny,nz) = rh(nx,ny,nz) + 1
        end do
       end subroutine DENSITY
C-----------------------------------------------------------------------
      subroutine DENSITY1(NTOT,NDIV,LENGTH,RHO,NEMAX,NE,RE)
C  calculate the average particle density RHO() on a cubic mesh
C  with NDIV intervals on each cubic axis, NDIV must be even
       implicit none
       integer,intent(in)     :: NTOT,NDIV,NEMAX,NE
       real(dp),intent(in)    :: LENGTH,RE(3,NEMAX)
       real(dp),intent(inout) :: RHO(NTOT,NTOT,NTOT)
C  local variables
       integer  :: nx,ny,nz,i
       real(dp) :: dl
       if (DBLE(NDIV/2) .ne. DBLE(NDIV)/2.D0) THEN
        write(*,*) 'NDIV not even: stop'
        stop
       end if
       dl = LENGTH/DBLE(NDIV)
       do i=1,NE
         nx = 1 + NDIV/2 + INT(RE(1,i)/dl)
         ny = 1 + NDIV/2 + INT(RE(2,i)/dl)
         nz = 1 + NDIV/2 + INT(RE(3,i)/dl)
         if ((nx .gt. NTOT) .or. (ny .gt. NTOT) .or.
     &       (nz .gt. NTOT)) THEN
          write(*,*)'too large nx, ny, or nz ',nx,ny,nz,' > NTOT= ',NTOT
          stop
         end if
         RHO(nx,ny,nz) = RHO(nx,ny,nz) + 1
       end do
      end subroutine DENSITY1
C-----------------------------------------------------------------------
      end module position
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      module output
C  output for 1-,2-,3-dimensional arrays on files termed PRONAME
C  with some additional naming
       use random
       implicit none
       public :: OUT3D
       integer,parameter,public :: FE1MAX=100,FE2MAX=100,FE3MAX=100
       integer :: ios
       character(7),public :: PRONAME
      contains
C
       subroutine OUT3D(NDIV1,NDIV2,NDIV3,a3d)
        integer :: NDIV1,NDIV2,NDIV3
        real(kind=dp),intent(in),dimension(FE1MAX,FE2MAX,FE3MAX)::
     &          a3d
        integer :: nx,ny,nz
        open(unit=36,file=PRONAME//"_erw10.dat",position="append",
     &               status="unknown")
        do nz=1,NDIV3
         do ny=1,NDIV2
          do nx=1,NDIV1
           write(36,55) nx,ny,nz,a3d(nx,ny,nz)
          end do
         end do
         write(36,*)'  '
         write(36,*)'  '
        end do
        close(36,iostat=ios)
        if (ios .gt. 0) then
          write(35,*) 'output error from OUT3D'
        end if
55     format(t3,i3,i3,i3,e12.3)
       end subroutine OUT3D
      end module output
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      program THERM
      use random
      use position
      use output,only: FE1MAX,FE2MAX,FE3MAX,PRONAME,OUT3D
C  NE = number of particles
      implicit none
      integer,parameter :: NXFA1=0,NXFA2=9
      real(dp)     :: X,Y,Z,STEPMAX,ALPHA,DALPHA,ALPHA0,ALPHA1,BETA
      real(dp)     :: DLENGTH,LENGTH0,LENGTH1,DUMMY
      real(dp)     :: RDIF,QUOT
      real(dp)     :: LOCEN,ERWEN,VAREN,ERWKIN,VARKIN,ERWPOT,VARPOT
      real(dp)     :: AVRHO(FE3MAX,FE3MAX,FE3MAX)
      logical      :: MCSCHRITT,SWIRHO
      character(7) :: FELDNAM
C  Local variables
      integer  :: n1,n2,i,k,ny,nz,M
      real(dp) :: RAD1,RAD2,RADNEU,EK,ep,rannumb
      real(dp) :: WORK1(NXFA2+1),WORK2(NXFA2+1),WORK
      logical  :: l
C
      PRONAME='thermod'
C   Controll output
      open(unit=35,file=PRONAME//'_MC10.OUT',position='append',
     &     status='unknown')
      write(35,*)'MC7 and MC6 have totally unbiased boundary condition'
      write (35,*)'totally unbiased is still biased;instead MC8 rejects'
      write (35,*)'any move outside the volume which is really unbiased'
C   Contour plot output  ALPHA and WAVEC for "xfarbe" tool
C   on a NXFA*NXFA grid
      open(unit=38,file=PRONAME//'erw_yukawa10.dat',position='append',
     &     status='unknown')
      open(unit=39,file=PRONAME//'var_yukawa10.dat',position='append',
     &     status='unknown')
C   Number of electrons and MC steps
      NE = 100
      IF (NE .GT. NEMAX) THEN
        write (*,*)'NE= ',NE,' larger than NEMAX= ',NEMAX
        STOP
      ENDIF
      MCPRE = 100000
      MCMAX = 1000000
      NDIV = 21    ! NDIV must be odd
      call INITRAN
C  Start data:
      lparam1:do m=1,3
      lparam2:do n1 = 1,NXFA1+1
      lparam3:do n2 = 1,NXFA2+1
C  Parameter ALPHA of inverse temperature and edge length LENGTH
      SWIRHO = .true.
      ALPHA0=0.50D0
      ALPHA1=10.0D0
      LENGTH0=+1.00D0
      LENGTH1=+50.0D0
      DALPHA=(ALPHA1-ALPHA0)/DBLE(NXFA1+1)
      DLENGTH=(LENGTH1-LENGTH0)/DBLE(NXFA2+1)
      ALPHA=ALPHA0+(n1-1)*DALPHA
C  adjust LENGTH to logarithmic plot
      LENGTH=(0.01+(n2-1)*0.01)*10**m
C      LENGTH=LENGTH0+(n2-1)*DLENGTH
C  Maximum step width, KONTUZ: Check with acceptance ratio!
      STEPMAX = 9.0D-2*LENGTH
C  Random initial electron positions
      do k=1,NE
        do i=1,3
          call GENRAN(rannumb)
          RE(i,k) =  LENGTH*(rannumb-0.5)
          RNEU(i,k) = RE(i,k)
        end do
C      write(*,*)'k= ',k,(RE(i,k),i=1,3)
      end do
      do i=1,NE
       VNEW(i) = 0.D0
       VDIF(i) = 0.D0
       do k=1,NE
        IF (k .eq. i) cycle
        DISTNEU(1:3,i,k) = RNEU(1:3,i)-RNEU(1:3,k)
        DIST(1:3,i,k) = DISTNEU(1:3,i,k)
        DISTNEU(4,i,k) = dsqrt(sum (DISTNEU(1:3,i,k)**2))
        DIST(4,i,k) = DISTNEU(4,i,k)
        VNEW(i) = VNEW(i) + 1.D0/DISTNEU(4,i,k)*dexp(-DISTNEU(4,i,k))
       end do
      end do
C  Counts the acceptance number
      MCOUNT = 0
C  Observables
      RHO(1:NDIV,1:NDIV,1:NDIV) = 0.D0
      AVRHO(1:NDIV,1:NDIV,1:NDIV) = 0.D0
      LOCEN = 0.D0
      ERWEN = 0.D0
      VAREN = 0.D0
      ERWKIN = 0.D0
      VARKIN = 0.D0
      ERWPOT = 0.D0
      VARPOT = 0.D0
      l = .true.
C
C
C  MC loop: prerun for thermalizing,KONTUZ: does not change the bad
C           sampling of energy!!!
      lrunpre:do IMC=2,MCPRE
       lelrunpre:do IE=1,NE
        do i=1,3
C  Shift position at random within +-STEPMAX/2
          call GENRAN(rannumb)
          RDIF = (rannumb-0.5)*STEPMAX
          RNEU(i,IE) = RE(i,IE)+RDIF
CCC  reflect particle at wall if it crosses
CC          if (RNEU(i,IE) > 0.5D0*LENGTH)
CC     )              RNEU(i,IE)=LENGTH-RNEU(i,IE)
CC          IF (RNEU(i,IE) < -0.5D0*LENGTH)
CC     )              RNEU(i,IE)=-LENGTH-RNEU(i,IE)
CCC  scatter particle at a rough wall
CC          if (DABS(RNEU(i,IE)) > 0.5D0*LENGTH) THEN
CC           call GENRAN(rannumb)
CC            if (RNEU(i,IE) > 0.5D0*LENGTH)
CC     &        RNEU(i,IE)=LENGTH/2.D0-ABS(rannumb-0.5)*STEPMAX
CC     &        RNEU(i,IE)=LENGTH*(1/2.D0-rannumb) ! totally unbiased
CC            if (RNEU(i,IE) < -0.5D0*LENGTH)
CC     &        RNEU(i,IE)=-LENGTH/2.D0+ABS(rannumb-0.5)*STEPMAX
CC     &        RNEU(i,IE)=-LENGTH*(1/2.D0-rannumb) ! totally unbiased
C   Apply strict cut-off boundary conditions
          if (DABS(RNEU(i,IE)) > 0.5D0*LENGTH) THEN
           MCSCHRITT = .false.
           RNEU(1:3,IE) = RE(1:3,IE)
           MCOUNT = MCOUNT + 1
           cycle lelrunpre
          end if
        end do
C  Calculate Boltzmann ratio with energy 0.5*sum_k 1/r_ik exp(-r_ik)
C    without term k=i
         call JASEXP(VNEW(IE),VDIF(IE))
         QUOT = DEXP(-ALPHA*VDIF(IE))
C  Test on acceptance
         if (QUOT < 1) THEN
C           MCSCHRITT = DBLE(ZBQLU01(DUMMY)) .LT. QUOT
           call GENRAN(rannumb)
           MCSCHRITT = DBLE(rannumb) < QUOT
         else
           MCSCHRITT = .true.
         end if
         if (MCSCHRITT) then
             RE(1:3,IE) = RNEU(1:3,IE)
             MCOUNT = MCOUNT + 1
         else
             RNEU(1:3,IE) = RE(1:3,IE)
         end if
       end do lelrunpre
      end do lrunpre
      write(35,*)'STEPMAX = ',STEPMAX
      write(35,*)'prerun: MCPRE= ',MCPRE,' acc. ratio = ',
     &           100.*DBLE(MCOUNT)/DBLE(NE*MCPRE),' %  '
      MCOUNT = 0
C
C
C
C  MC loop: main run after thermalizing
      lrun:do IMC=2,MCMAX
       lelrun:do IE=1,NE
        do i=1,3
C  Shift position at random within +-STEPMAX/2
          call GENRAN(rannumb)
          RDIF = (rannumb-0.5)*STEPMAX
          RNEU(i,IE) = RE(i,IE)+RDIF
CCC  Reflect particle at wall if it crosses
CC          if (RNEU(i,IE) > 0.5D0*LENGTH)
CC     )              RNEU(i,IE)=LENGTH-RNEU(i,IE)
CC          if (RNEU(i,IE) < -0.5D0*LENGTH)
CC     )              RNEU(i,IE)=-LENGTH-RNEU(i,IE)
CCC  Scatter particle at a rough wall
CC          if (DABS(RNEU(i,IE)) .gt. 0.5D0*LENGTH) then
CC            call GENRAN(rannumb)
CC            if (RNEU(i,IE) > 0.5D0*LENGTH)
CC     &        RNEU(i,IE)=LENGTH/2.D0-ABS(ZBQLU01(DUMMY)-0.5)*STEPMAX
CC            if (RNEU(i,IE) < -0.5D0*LENGTH)
CC     &        RNEU(i,IE)=-LENGTH/2.D0+ABS(ZBQLU01(DUMMY)-0.5)*STEPMAX
CC     &        RNEU(i,IE)=LENGTH/2.D0-ABS(rannumb-0.5)*STEPMAX
CC     &        RNEU(i,IE)=LENGTH*(1/2.D0-rannumb) ! totally unbiased
CC            if (RNEU(i,IE) .lt. -0.5D0*LENGTH)
CC     &        RNEU(i,IE)=-LENGTH/2.D0+ABS(rannumb-0.5)*STEPMAX
CC     &        RNEU(i,IE)=-LENGTH*(1/2.D0-rannumb) ! totally unbiased
C   Apply strict cut-off boundary conditions
          if (DABS(RNEU(i,IE)) > 0.5D0*LENGTH) THEN
            MCSCHRITT = .false.
            l = .false.
            exit
          end if
        end do
        if (l) then
C  Calculate Boltzmann ratio with energy 0.5*sum_k 1/r_ik exp(-r_ik)
C    without term k=i
         call JASEXP(VNEW(IE),VDIF(IE))
         QUOT = DEXP(-ALPHA*VDIF(IE))
C  Test on acceptance
         if (QUOT < 1) then
           call GENRAN(rannumb)
           MCSCHRITT = DBLE(rannumb) < QUOT
         else
           MCSCHRITT = .true.
         end if
        end if
        l = .true.
C  Update of observables.
        if (MCSCHRITT) then
          RE(1:3,IE) = RNEU(1:3,IE)
          ep = VNEW(IE)
          MCOUNT = MCOUNT + 1
        else
          RNEU(1:3,IE) =  RE(1:3,IE)
          ep = VNEW(IE) - VDIF(IE)
        end if
C  E=0.5 sum_ik v_ik, sum i omitted because
C  run averages over particles, so energy per particle is calculated
        LOCEN = LOCEN + 0.5D0*ep
       end do lelrun
C  energy per particle
       LOCEN = LOCEN/DBLE(NE)
       if (IMC == 2) then  !set start values
        ERWEN = LOCEN
        VAREN = 0.D0
       end if
       ERWEN = (IMC-1)/DBLE(IMC)*ERWEN+LOCEN/DBLE(IMC)
       VAREN = (IMC-1)/DBLE(IMC)*VAREN +
     &          1/DBLE(IMC-1)*(ERWEN-LOCEN)**2
       LOCEN = 0.D0
       if (SWIRHO) then
C  density
        call DENSITY(RHO)
        do nz=1,NDIV
         do ny=1,NDIV
          AVRHO(1:NDIV,ny,nz)=(IMC-1)/DBLE(IMC)*AVRHO(1:NDIV,ny,nz)
     &                   +  RHO(1:NDIV,ny,nz)/DBLE(IMC)
         end do
        end do
       end if
      end do lrun
C  end MC loop
C
      WORK1(n2) = ERWEN
      WORK2(n2) = VAREN
      write(35,35)'main run: MCMAX= ',MCMAX,' acc. ratio = ',
     ,           100.*DBLE(MCOUNT)/DBLE(NE*MCMAX),' %  '
      write(35,*)'ALPHA = ',ALPHA,' LENGTH = ',LENGTH
      write(35,*)'energy+0.5*ALPHA**2 = ',ERWEN+0.5*ALPHA**2
      write(35,*)'ERWEN = ',ERWEN,'  VAREN = ',VAREN
      write(35,*)
      write(35,*)
      if (SWIRHO) then
        write (36,*) 'ALPHA = ',ALPHA,' LENGTH = ',LENGTH
        call OUT3D(NDIV,NDIV,NDIV,AVRHO)
      end if
      write(38,45) LENGTH,ERWEN
      write(39,45) LENGTH,VAREN
      end do lparam3
      write(38,fmt="(t3)")
      write(39,fmt="(t3)")
      end do lparam2
      end do lparam1
      CLOSE(35)
      CLOSE(38)
      CLOSE(39)
35    FORMAT(1X,A,I8,A,F7.3,A)
45    FORMAT(t3,F7.3,E12.3)
      end program THERM
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
